# -*- coding: utf-8 -*-
# moebel.py

from grafikfenster import *
from math import radians

from abc import * # abstrakte Klasse realisieren

import pickle


### ---------------------------------------------------
class Moebel(ABC):
    """Klasse Moebel
    Oberklasse für das Zeichnen und Bearbeiten von
    Möbel-Symbolen fuer den Raumplaner"""

    __alleMoebel = []
    __clickPosition = None
    __ausgewaehlt = -1
    __gui = None # Gui ist Beobachter bei Auswahl

    @staticmethod
    def SetzeGui(gui):
        '''Gui ist Beobachter bei Auswahl'''
        Moebel.__gui=gui

    @staticmethod
    def FuegeHinzu(moebel):
        '''Fuegt der Moebelliste ein neues hinzu'''
        if moebel in Moebel.__alleMoebel: return
        Moebel.__alleMoebel.append(moebel)

    #### ----- Auswahl -----
    @staticmethod
    def Waehle(index=None):
        '''wählt das Moebel aus;
        None zum Weitersetzen der Auswahl, -1 zum Beenden der Auswahl,
        nach dem letzten wird die Auswahl ebenfalls beendet'''
        if len(Moebel.__alleMoebel)==0: # nichts auszuwaehlen
            return
        if index==None: # Weitersetzen
            if Moebel.__ausgewaehlt!=-1:
                Moebel.__alleMoebel[Moebel.__ausgewaehlt].WaehleAus(False)
                Moebel.__alleMoebel[Moebel.__ausgewaehlt].Zeige()                
            Moebel.__ausgewaehlt+=1
            if Moebel.__ausgewaehlt==len(Moebel.__alleMoebel):
                Moebel.__ausgewaehlt=-1
            else:
                Moebel.__alleMoebel[Moebel.__ausgewaehlt].WaehleAus(True)
                Moebel.__alleMoebel[Moebel.__ausgewaehlt].Zeige()
            Moebel.__gui.UpdateDaten()
            return
        # Indexfehler pruefen
        if index<-1: index=-1
        if index>=len(Moebel.__alleMoebel): index=-1
        if Moebel.__ausgewaehlt!=-1: # bisherige Auswahl beenden
            Moebel.__alleMoebel[Moebel.__ausgewaehlt].WaehleAus(False)
            Moebel.__alleMoebel[Moebel.__ausgewaehlt].Zeige()
        if index==-1: return # Auswahl ist geloescht
        Moebel.__ausgewaehlt=index # Darstellung aktualisieren
        Moebel.__alleMoebel[index].WaehleAus(True)
        Moebel.__gui.UpdateDaten()

    @staticmethod
    def LoescheAusgewaehltes():
        if Moebel.__ausgewaehlt>=0:
            Moebel.GibAusgewaehltes().Verberge()
            Moebel.__alleMoebel.remove(Moebel.GibAusgewaehltes())
            Moebel.__ausgewaehlt=-1

    @staticmethod
    def GibAusgewaehltes():
        if Moebel.__ausgewaehlt<0: return None
        return Moebel.__alleMoebel[Moebel.__ausgewaehlt]

    @staticmethod
    def GibAktuelleDaten():
        '''fuer die DArstellung in der Gui'''
        if Moebel.__ausgewaehlt<0: return ('','','','')
        aktuell=Moebel.GibAusgewaehltes()
        daten=(aktuell.GibXPosition(),
               aktuell.GibYPosition(),
               aktuell.GibOrientierung(),
               aktuell.GibFarbe())
        return daten

    @staticmethod
    def BearbeiteAusgewaehltes(**kwargs):
        """die kwargs dx,dy,dw,f erzeugen ein Dictionary"""
        if Moebel.__ausgewaehlt>=0:
            gewaehlt=Moebel.GibAusgewaehltes()
            if 'dx' in kwargs:
                gewaehlt.BewegeHorizontal(int(kwargs['dx']))
            if 'dy' in kwargs:
                gewaehlt.BewegeVertikal(int(kwargs['dy']))
            if 'dw' in kwargs:
                gewaehlt.Drehe(int(kwargs['dw']))
            if 'f' in kwargs:
                gewaehlt.AendereFarbe(kwargs['f'])

    @staticmethod
    def GibAlleMoebel():
        '''Gibt die Liste aller erzeugten Moebel zurueck'''
        return Moebel.__alleMoebel

    @staticmethod
    def Oeffnen(dateiname='Daten/daten.dat'):
        ## erst alle vorhandenen loeschen
        for moebel in Moebel.__alleMoebel:
            moebel.Verberge()
            Moebel.Entferne(moebel)
        try:
            with open(dateiname, 'rb') as objekt_datei:
                Moebel.__alleMoebel = pickle.load(objekt_datei)
                for moebel in Moebel.__alleMoebel: moebel.Zeige()
        except IOError:
            print('Datei nicht auffindbar!')
        except ImportError as impErr:
            print('Import-Fehler: ' + str(impErr))
        except pickle.UnpicklingError as perr:
            print('Pickle-Fehler: ' + str(perr))
        
    @staticmethod
    def Speichern(dateiname='Daten/daten.dat'):
        try:
            with open(dateiname, 'wb') as objekt_datei:
                pickle.dump(Moebel.__alleMoebel, objekt_datei)
        except IOError as ioerr:
            print('Dateifehler: ' + str(ioerr))
        except pickle.PicklingError as perr:
            print('Pickle-Fehler: ' + str(perr))

    @staticmethod
    def BildExport(dateiname='Bilder/Bild.png'):
        """Bild wird in Bild-Datei geschrieben"""
        Zeichenflaeche.GibZeichenflaeche().BildExport(dateiname)        
    
    ##### Methode fuer die Verarbeitung der Position:
    @staticmethod
    def Angeklickt(moebel, x, y):        
        pos=wx.Point2D(x,y)
        return moebel.GibFigur().GetBox().Contains(pos)
    
    ##### Methoden fuer die Ereignisbehandlung:
    @staticmethod
    def ClickAuswahl(x,y):
        '''bestimmt das zur Position passende Moebel;
        wird keines gefunden, wird die Auswahl gelöscht'''
        Moebel.Waehle(-1)
        for moebel in Moebel.__alleMoebel:
            if Moebel.Angeklickt(moebel,x,y):
                moebel.WaehleAus(True)
                Moebel.__ausgewaehlt = Moebel.__alleMoebel.index(moebel)
                Moebel.Waehle(Moebel.__ausgewaehlt)
                return
    
    @staticmethod
    def MausClick(x,y):
        '''verarbeitet den Klick mit der linken Maustaste'''
        Moebel.ClickAuswahl(x,y)
        if Moebel.__ausgewaehlt<0: Moebel.__clickPosition=None
        else: Moebel.__clickPosition=(x,y)

    @staticmethod
    def MausLoslassen(x,y):
        if Moebel.__ausgewaehlt == -1 : return
        Moebel.__clickPosition=None
        Moebel.__gui.UpdateDaten()
        
    @staticmethod
    def MausGezogen(x,y):
        if Moebel.__ausgewaehlt == -1 : return
        dX=x-Moebel.__clickPosition[0]
        dY=y-Moebel.__clickPosition[1]
        Moebel.__alleMoebel[Moebel.__ausgewaehlt].Bewege(dX,dY)
        Moebel.__clickPosition=(x,y)
        Moebel.__gui.UpdateDaten()

    @staticmethod
    def DoppelKlickLinks(x,y):
        '''nicht implementiert'''
        pass
        
    @staticmethod
    def MausBewegt(x,y):
        '''nicht implementiert'''
        pass
    
    @staticmethod
    def MausRechtsClick(x,y):
        '''nicht implementiert'''
        pass

#### ----- Konstruktor --------------------------------
    def __init__(self,
                 xPos,
                 yPos,
                 breite,
                 tiefe,
                 winkel,
                 farbe,
                 sichtbar):
        """Konstruktor mit vordefinierten Parametern
        ist bei Moebel nicht sinnvoll"""
        self.__gewaehlt=False
        self.__x=xPos
        self.__y=yPos
        self.__b=breite
        self.__t=tiefe
        self.__w=winkel
        self.__f=farbe
        self.__s=sichtbar
        Moebel.FuegeHinzu(self)
        if sichtbar: self.Zeige()

    @abstractmethod
    def GibFigur(self):
        """Definiert die zu zeichnende Figur."""
        return NotImplemented

    def GibZeichenPfad(self):
        return Zeichenflaeche.GibZeichenflaeche().GibGC().CreatePath()

    def GibTransformation(self):
        """erzeugt die zugehörige Transformation"""
        gc = Zeichenflaeche.GibZeichenflaeche().GibGC()
        matrix=gc.CreateMatrix()
        matrix.Translate(self.__x+self.__b/2, self.__y+self.__t/2)
        matrix.Rotate(radians(self.__w))
        matrix.Translate(-self.__b/2, -self.__t/2)
        return matrix

    def Transformiere(self, path):
        """Transformiert den übergebenen Pfad"""
        path.Transform(self.GibTransformation())
        return path

    def GibXPosition(self):
        """Get-Methode fuer die xPosition"""
        return self.__x
    
    def GibYPosition(self):
        """Get-Methode fuer die yPosition"""
        return self.__y
    
    def GibBreite(self):
        """Get-Methode fuer die Breite"""
        return self.__b
    
    def GibTiefe(self):
        """Get-Methode fuer die Tiefe"""
        return self.__t
    
    def GibOrientierung(self):
        """Get-Methode fuer die Orientierung"""
        return self.__w
    
    def GibFarbe(self):
        """Get-Methode fuer die Farbe"""
        if self.__gewaehlt: return "gray"
        return self.__f
    
    def GibSichtbar(self):
        """Get-Methode fuer die Sichtbarkeit"""
        return self.__s

    def Bewege(self, xWeite, yWeite):
        """Veraendernde Methode fuer die x- und y-Position"""
        self.Loesche()
        self.__x += xWeite
        self.__y += yWeite
        self.Zeichne()
    
    def BewegeHorizontal(self, weite):
        """Veraendernde Methode fuer die x-Position"""
        self.Loesche()
        self.__x += weite
        self.Zeichne()
    
    def BewegeVertikal(self, weite):
        """Veraendernde Methode fuer die y-Position"""
        self.Loesche()
        self.__y += weite
        self.Zeichne()
    
    def Drehe(self, winkel):
        """Veraendernde Methode fuer die Orientierung [Winkel]"""
        self.Loesche()
        self.__w += winkel
        self.Zeichne()

    def WaehleAus(self, auswaehlen): # Umbenannt wegen Klassenmethode
        if auswaehlen: self.__gewaehlt=True
        else: self.__gewaehlt=False

    def Verberge(self):
        """Veraendernde Methode fuer die Sichtbarkeit mit Wert False"""
        self.__s = False
        self.Loesche()

    def Zeige(self):
        """Veraendernde Methode fuer die Sichtbarkeit mit Wert True"""
        self.__s = True
        self.Zeichne()

    def Loesche(self):
        """Entfernt ein Objekt aus der Darstellung."""
        Zeichenflaeche.GibZeichenflaeche().Entferne(self)

    def Zeichne(self):
        """Fuegt das Objekt den darzustellenden hinzu."""
        Zeichenflaeche.GibZeichenflaeche().Zeichne(self)

### ---------------------------------------------------
class MoebelApp(wx.App):
    """Test-Anwendung speziell fuer Stuhl"""
    def OnInit(self):
        self.fenster = GrafikFenster(None, "Raumplaner-Grafik")
        self.SetTopWindow(self.fenster)
        self.fenster.Show(True)
        self.fenster.panel.Refresh()
        self.fenster.ZeigeShellFrame()
        #self.fenster.ZeigeFillingFrame()
        self.TestAnwendung()
        return True

    def TestAnwendung(self):
        """Allein fuer die Testanwendung:
        sollte nicht gehen!"""
        global moebel
        try:
            moebel=Moebel(70, 30, 120, 60, 0, 'red', True)
        except Exception as e:
            print( str(e) )
        
### ---------------------------------------------------
if __name__ == '__main__':
    app = MoebelApp(redirect=False)
    # Parameterwert True, wenn Ausgaben in der Standard E/A angezeigt werden sollen
    app.MainLoop()
